import logging
from odoo import models, fields, api, _
from odoo.osv.expression import AND
from odoo.exceptions import ValidationError, UserError
from datetime import date, timedelta

_logger = logging.getLogger(__name__)


class PosOrder(models.Model):
    _inherit = "pos.order"

    l10n_latam_document_number = fields.Char(
        related="account_move.l10n_latam_document_number",
        store=True,
        string="Comprobante fiscal",
    )
    l10n_latam_document_type_id = fields.Many2one(
        related="account_move.l10n_latam_document_type_id",
        readonly=True,
        store=True,
    )
    l10n_do_fiscal_number = fields.Char("NCF", copy=False,)
    l10n_do_origin_ncf = fields.Char(string="NCF origen", copy=False,)
    l10n_do_ncf_expiration_date = fields.Date(
        related="l10n_latam_document_type_id.l10n_do_ncf_expiration_date",
    )
    l10n_latam_use_documents = fields.Boolean()
    l10n_latam_country_code = fields.Char(
        related="company_id.country_id.code",
        index=True,
        store=True,
        help="Technical field used to hide/show fields regarding the localization",
    )
    l10n_do_is_return_order = fields.Boolean(
        string="Es una devolución",
        copy=False,
    )
    display_name = fields.Char(compute="_compute_new_display_name", store=True, index=True)

    @api.depends("pos_reference", "l10n_latam_document_number")
    def _compute_new_display_name(self):
        for record in self:
            record.display_name = "%s - %s" % (record.pos_reference, record.l10n_latam_document_number)

    @api.depends()
    def _compute_refund_related_fields(self):
        super()._compute_refund_related_fields()
        for order in self:
            order.l10n_do_origin_ncf = order.refunded_order_id.l10n_latam_document_number
            
    @api.model_create_multi
    def create(self, vals_list):
        for vals in vals_list:
            vals.update({
                'l10n_do_fiscal_number': vals.get('l10n_latam_document_number', False),
            })
        return super().create(vals_list)

    # @api.model
    # def sync_from_ui(self, orders):
    #     data = super().sync_from_ui(orders)
    #     _logger.info(f"sync_from_ui: {data}")
    #     for order in data['pos.order']:
    #         pos_order = self.browse(order.get('id'))
    #         _logger.info(f"Pedido: {pos_order}")
    #         _logger.info(f"Datos: {order}")
    #         # _logger.info(f"Datos: {porder}")
    #         data['l10n_do_fiscal_number'] = pos_order.l10n_latam_document_number
    #         # data['l10n_latam_document_type_id'] = pos_order.l10n_latam_document_type_id.name
    #         # data['l10n_do_origin_ncf'] = pos_order.l10n_do_origin_ncf
    #         # data['l10n_do_ecf_modification_code'] = pos_order.l10n_do_ecf_modification_code
    #
    #     return data

    def _prepare_invoice_vals(self):
        """
        Prepare the dict of values to create the new invoice for a pos order.
        generate_pdf
        """
        invoice_vals = super(PosOrder, self)._prepare_invoice_vals()
        if invoice_vals['move_type'] == 'out_refund':
            invoice_vals['ref'] = self.l10n_do_fiscal_number
            invoice_vals['l10n_do_origin_ncf'] = self.l10n_do_origin_ncf
            # invoice_vals['l10n_do_ecf_modification_code'] = self.l10n_do_ecf_modification_code

        return invoice_vals

    # @api.model
    # def _payment_fields(self, order, ui_paymentline):
    #     fields = super()._payment_fields(order, ui_paymentline)
    #     payment_note = ui_paymentline.get('note', False)
    #     fields.update({
    #         'name': payment_note or ui_paymentline.get('name', False) or '',
    #         'note': payment_note,
    #     })
    #     return fields

    def _generate_pos_order_invoice(self):
        # Añadir generate_pdf=False al contexto
        self = self.with_context(generate_pdf=False)
        # Llamar al método original
        return super(PosOrder, self)._generate_pos_order_invoice()

    @api.model
    def get_credit_note(self, fiscal_number, partner_id):
        """
        RPC llamado desde el POS:
        - Devuelve {} si no existe o su residual es 0.
        - Si existe: dict con move_id, partner_id, residual_amount, fiscal_number.
        """
        note = self.env["account.move"].search(
            [
                ("partner_id", "=", partner_id),
                ("l10n_do_fiscal_number", "=", fiscal_number),
                ("move_type", "=", "out_refund"),
                ("l10n_latam_use_documents", "=", True),
                ("company_id", "=", self.env.company.id),
                ("state", "=", "posted"),
            ],
            limit=1,
        )

        if not note or not note.amount_residual:
            return {}

        return {
            "move_id": note.id,
            "partner_id": note.partner_id.id,
            "residual_amount": note.amount_residual,
            "fiscal_number": note.l10n_do_fiscal_number,
        }

        # Lista vacía si no hay resultados; nunca devolvemos 'error'
        return [
            {
                "id": mv.id,
                "label": f"{mv.l10n_do_fiscal_number} • {mv.amount_residual:.2f}",
                "item": {
                    "move_id": mv.id,
                    "partner_id": mv.partner_id.id,
                    "residual_amount": mv.amount_residual,
                    "fiscal_number": mv.l10n_do_fiscal_number,
                },
            }
            for mv in moves
        ]
